@@ -27,8 +27,8 @@ def priv2bits (priv_mode):
 def csr_gen (isa_file, grp_file, custom_file, debug_file, custom_attr_file, work_dir,
         probeinst, logging=False):
     '''
-        Top level function which validates the sanity of the grouping yaml 
-        followed by calls to subfunctions which generated the corresponding 
+        Top level function which validates the sanity of the grouping yaml
+        followed by calls to subfunctions which generated the corresponding
         bluespec code. This function further assumes that the riscv-config
         isa yaml provided is the validated/normalized version.
 
@@ -44,7 +44,7 @@ def csr_gen (isa_file, grp_file, custom_file, debug_file, custom_attr_file, work
         :type grp_file: str
         :type work_dir: str
     '''
-    
+
     isa_yaml = utils.load_yaml(isa_file)
     grp_yaml = utils.load_yaml(grp_file)
     global debug_enabled
@@ -64,7 +64,7 @@ def csr_gen (isa_file, grp_file, custom_file, debug_file, custom_attr_file, work
         supervisor_enabled = True
     else:
         supervisor_enabled = False
-    
+
     if 'H' in isa_yaml['hart0']['ISA']:
         hypervisor_enabled = True
     else:
@@ -88,14 +88,14 @@ def csr_gen (isa_file, grp_file, custom_file, debug_file, custom_attr_file, work
     if supervisor_enabled:
         satp_modewarl =\
                     (warl_interpreter(isa_yaml['hart0']['satp']['rv'+str(xlen)]['mode']['type']['warl']))
-        for x in [0, 1, 8, 9]:
+        for x in [0, 1, 8, 9, 10]:
             if satp_modewarl.islegal(x,[]):
                 if xlen == 64:
                     valid_satp_modes.append('word[63:60] == ' +str(x))
                 else:
                     valid_satp_modes.append('word[31] == ' +str(x))
     satp_mode_condition = '||'.join(valid_satp_modes)
-    
+
     csrgrp_check.csrgrp_check(isa_yaml, grp_yaml, logging)
     attr_yaml = csrgrp_check.calculate_size(isa_yaml, custom_attr_file)
     csr_define(isa_yaml , work_dir , 'csrbox.defines', logging)
@@ -107,21 +107,21 @@ def csr_types(isa_yaml, work_dir,  bsv, logging=False):
     '''
         Function to generate the types bluespec file which includes structures
         and functions that are used across the csrbox modules
-        
+
         :param isa_yaml: the riscv-config isa YAML
         :param work_dir: path to the directory where the bluespec files should be generated
         :param bsv: name of the file to be created inside the work_dir
 
-        :type isa_yaml: dict 
+        :type isa_yaml: dict
         :type work_dir: str
         :type bsv: str
 
     '''
-    if logging: 
+    if logging:
         logger.info('Creating File: ' + bsv)
     bsvfile = open(work_dir + bsv, 'w')
     pkg_name = bsv[:-4]
-    
+
     xlen = str(isa_yaml['hart0']['supported_xlen'][0])
 
     bsvfile.write(license_header.format(version, str(datetime.datetime.now())))
@@ -162,10 +162,10 @@ def groups_gen(isa_yaml, grp_yaml, attr_yaml, work_dir, probeinst, logging=False
         :param grp_yaml: the grouping YAML
         :param attr_yaml: the attributes YAML
         :param work_dir: path to the directory where the bluespec files should be generated
-        
-        :type isa_yaml: dict 
-        :type grp_yaml: dict 
-        :type attr_yaml: dict 
+
+        :type isa_yaml: dict
+        :type grp_yaml: dict
+        :type attr_yaml: dict
         :type work_dir: str
     '''
 
@@ -225,8 +225,8 @@ def groups_gen(isa_yaml, grp_yaml, attr_yaml, work_dir, probeinst, logging=False
         bsvstr += (end_module_temp)
         bsvstr +=(end_package_temp)
         bsvfile.write(bsvstr)
-        station_isa.clear()  
-    bsvstr = csr_top(csr_isa, grp_yaml, attr_yaml, aux_info, rvxlen, 
+        station_isa.clear()
+    bsvstr = csr_top(csr_isa, grp_yaml, attr_yaml, aux_info, rvxlen,
              'csrbox', logging)
     filename = 'csrbox.bsv'
     bsvfile = open(work_dir + filename, 'w')
@@ -258,19 +258,19 @@ def csr_probe_function(isa_yaml, attr_yaml,probeinst, logging=False):
   endfunction
 '''
     return bsvstr
-    
+
 
 
 def csr_decoder(isa_yaml, work_dir, bsv, logging=False):
 
-    ''' 
+    '''
         Function to generate the bluespec decoder file indicating if the access
         to the CSR if valid or not
         :param isa_yaml: the riscv-config isa YAML
         :param work_dir: path to the directory where the bluespec files should be generated
         :param bsv: name of the file to be created inside the work_dir
 
-        :type isa_yaml: dict 
+        :type isa_yaml: dict
         :type work_dir: str
         :type bsv: str
     '''
@@ -324,7 +324,7 @@ def csr_define(isa_yaml, work_dir, bsv, logging=False):
         :param work_dir: path to the directory where the bluespec files should be generated
         :param bsv: name of the file to be created inside the work_dir
 
-        :type isa_yaml: dict 
+        :type isa_yaml: dict
         :type work_dir: str
         :type bsv: str
     '''
@@ -342,12 +342,12 @@ def csr_define(isa_yaml, work_dir, bsv, logging=False):
                 csr_name = node
                 if logging:
                  logger.debug('Processing CSR: ' + str(csr_name.upper()))
-                
+
                 csr_address = format(isa_yaml['hart0'][csr_name]['address'],'x')
 
                 if logging:
                  logger.debug('|-- address: ' + str(csr_address))
-                 
+
                 definefile.write(address_temp.format(csr_name.upper(),csr_address))
 
     if isa_yaml['hart0']['custom_exceptions'] is not None:
@@ -399,7 +399,7 @@ def create_concat_vector(mylist, xlen):
 
 
 def get_reset_val(value, width, lsb, xlen):
-    ''' 
+    '''
     Function to extract 'width' number of bits starting at the 'lsb' position of
     the value. Note here that the python indexes the binary string from the let
     while the lsb defines the position from the right. Hence the need for this
@@ -441,7 +441,7 @@ def derive_wlrl_func(wlrl_list):
         full_wlrl = False
 
     return full_wlrl, act_width
-    
+
 
 def derive_warl_func(field_name, width, warl, csr_dep, post_op=''):
     '''
@@ -489,7 +489,7 @@ def derive_warl_func(field_name, width, warl, csr_dep, post_op=''):
                 condition_str += ' {0}[{1}] {2} {3}'.format(csr_write, indices, op, val)
                 condition_str += ' && '
             condition_str = condition_str[:-4] + ' )begin\n                  '
-        
+
 
 
         range_vals = regex_range.findall(legal)
@@ -538,7 +538,7 @@ def derive_warl_func(field_name, width, warl, csr_dep, post_op=''):
             if post_op != '':
                 curr_bsv_str = curr_bsv_str + post_op + ';'
             curr_bsv_str = curr_bsv_str + 'r._write(_x);'
-        if condition_str != '': 
+        if condition_str != '':
             curr_bsv_str += ' end\n'
         bsv_str += condition_str + curr_bsv_str
         bsv_str += '                else  '
@@ -550,7 +550,7 @@ def station_interface(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkg_name,
 
     '''
         Function to create the interface definition of individual
-        groups/stations. 
+        groups/stations.
 
         First Value methods are created for all csrs that are instantiated in
         the particular group. Next, custom action methods required for specific
@@ -566,10 +566,10 @@ def station_interface(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkg_name,
         :param last: indicating this is the last station in the chain
         :param bsvstr: string containing rest of the code generated for the particular group/station
         :param work_dir: path to the directory where the bluespec files should be generated
-        
-        :type isa_yaml: dict 
-        :type grp_yaml: dict 
-        :type attr_yaml: dict 
+
+        :type isa_yaml: dict
+        :type grp_yaml: dict
+        :type attr_yaml: dict
         :type rvxlen: str
         :type pkg_name: str
         :type last: bool
@@ -580,7 +580,7 @@ def station_interface(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkg_name,
     global compressed_enabled
     global supervisor_enabled
     global hypervisor_enabled
-    global physical_addr_sz 
+    global physical_addr_sz
     xlen = int(rvxlen[2:])
     bsvstr += interface_temp.format(pkg_name)
 
@@ -623,7 +623,7 @@ def station_interface(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkg_name,
                 arguments = arguments.replace('size',str(size))
                 if '::' in arguments:
                     cond, args = re.findall(r'^(.*?)::(.*?)$',arguments)[0]
-                    if eval(cond):         
+                    if eval(cond):
                         args = textwrap.indent(args, '')
                     else:
                         args = ''
@@ -696,9 +696,9 @@ def station_interface(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkg_name,
 def station_module(isa_yaml, grp_yaml, attr_yaml, rvxlen, pkg_name,
         last, bsvstr, logging = False):
 
-    ''' 
-        Function to create the bluespec module syntax and other synthesis 
-        attributes of the module 
+    '''
+        Function to create the bluespec module syntax and other synthesis
+        attributes of the module
 
         :param isa_yaml: the riscv-config isa YAML
         :param grp_yaml: the grouping YAML
@@ -708,10 +708,10 @@ def station_module(isa_yaml, grp_yaml, attr_yaml, rvxlen, pkg_name,
         :param last: indicating this is the last station in the chain
         :param bsvstr: string containing rest of the code generated for the particular group/station
         :param work_dir: path to the directory where the bluespec files should be generated
-        
-        :type isa_yaml: dict 
-        :type grp_yaml: dict 
-        :type attr_yaml: dict 
+
+        :type isa_yaml: dict
+        :type grp_yaml: dict
+        :type attr_yaml: dict
         :type rvxlen: str
         :type pkg_name: str
         :type last: bool
@@ -729,7 +729,7 @@ def station_module(isa_yaml, grp_yaml, attr_yaml, rvxlen, pkg_name,
                if '::' in modattr :
                         condition, _modattr = re.findall(r'^(.*?)::(.*?)$',modattr)[0]
                         if eval(condition) :
-                           attr_str += _modattr +'\n' 
+                           attr_str += _modattr +'\n'
                else:
                 attr_str += '  ' + modattr + '\n'
     bsvstr += (module_temp.format(pkg_name, attr_str, ''))
@@ -740,7 +740,7 @@ def station_module(isa_yaml, grp_yaml, attr_yaml, rvxlen, pkg_name,
 
 def station_modbody (isa_yaml, grp_yaml, attr_yaml, rvxlen, pkg_name,
         last, bsvstr, logging = False):
-    ''' 
+    '''
         Function to create the bluespec module rules/body requirements per csr
         basis
 
@@ -752,10 +752,10 @@ def station_modbody (isa_yaml, grp_yaml, attr_yaml, rvxlen, pkg_name,
         :param last: indicating this is the last station in the chain
         :param bsvstr: string containing rest of the code generated for the particular group/station
         :param work_dir: path to the directory where the bluespec files should be generated
-        
-        :type isa_yaml: dict 
-        :type grp_yaml: dict 
-        :type attr_yaml: dict 
+
+        :type isa_yaml: dict
+        :type grp_yaml: dict
+        :type attr_yaml: dict
         :type rvxlen: str
         :type pkg_name: str
         :type last: bool
@@ -774,7 +774,7 @@ def station_modbody (isa_yaml, grp_yaml, attr_yaml, rvxlen, pkg_name,
 
 def station_methods(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkg_name,
         last, bsvstr, logging = False):
-    ''' 
+    '''
         Function to create the bluespec method definitions for each
         group/station. First the value methods of each csr in the group is
         created. Followed by custom action method and then the shadow/dependent
@@ -788,10 +788,10 @@ def station_methods(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkg_name,
         :param last: indicating this is the last station in the chain
         :param bsvstr: string containing rest of the code generated for the particular group/station
         :param work_dir: path to the directory where the bluespec files should be generated
-        
-        :type isa_yaml: dict 
-        :type grp_yaml: dict 
-        :type attr_yaml: dict 
+
+        :type isa_yaml: dict
+        :type grp_yaml: dict
+        :type attr_yaml: dict
         :type rvxlen: str
         :type pkg_name: str
         :type last: bool
@@ -804,7 +804,7 @@ def station_methods(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkg_name,
     xlen = int(rvxlen[2:])
     if logging:
         logger.info('Creating method definitions for : ' + pkg_name)
-    
+
     # default value methods
     for csrname in isa_yaml:
         if logging:
@@ -824,7 +824,7 @@ def station_methods(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkg_name,
                         bsvstr += methodv_def_temp.format(csrname,'',_str)
             else:
                 bsvstr += methodv_def_temp.format(csrname, 'rg_', csrname)
-    
+
     # custom action methods
     for csrname in isa_yaml:
         if logging:
@@ -841,7 +841,7 @@ def station_methods(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkg_name,
                     else:
                         _str = ''
                 bsvstr += textwrap.indent( _str+ '\n', '    ')
-    
+
     # create action methods of dependent csrs
     incomming_reads = []
     for csrname in isa_yaml:
@@ -924,7 +924,7 @@ def station_methods(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkg_name,
 
 def station_req(isa_yaml, grp_yaml, attr_yaml, rvxlen, pkg_name,
         last, bsvstr, logging = False):
-    ''' 
+    '''
         The function to create the final methods which serves the request from
         the core for each csr.
 
@@ -936,10 +936,10 @@ def station_req(isa_yaml, grp_yaml, attr_yaml, rvxlen, pkg_name,
         :param last: indicating this is the last station in the chain
         :param bsvstr: string containing rest of the code generated for the particular group/station
         :param work_dir: path to the directory where the bluespec files should be generated
-        
-        :type isa_yaml: dict 
-        :type grp_yaml: dict 
-        :type attr_yaml: dict 
+
+        :type isa_yaml: dict
+        :type grp_yaml: dict
+        :type attr_yaml: dict
         :type rvxlen: str
         :type pkg_name: str
         :type last: bool
@@ -981,7 +981,7 @@ def station_req(isa_yaml, grp_yaml, attr_yaml, rvxlen, pkg_name,
         else:
             write_cond = ''
 
-        if is_shadow: 
+        if is_shadow:
             regname = shadow_csr.replace('.','_')
         logger.debug('Case statement for ' +csrname + ' Address: ' +\
                 str(address))
@@ -1012,7 +1012,7 @@ def station_req(isa_yaml, grp_yaml, attr_yaml, rvxlen, pkg_name,
             bsvstr += case_def.format(csrname.upper(), regname, xlen, write_cond)
             if logging:
                 logger.debug('-- Creating default logic for : ' + csrname)
-    
+
     if last:
         bsvstr += (endcase_temp)
     else:
@@ -1040,10 +1040,10 @@ def station_regs(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkg_name,
         :param last: indicating this is the last station in the chain
         :param bsvstr: string containing rest of the code generated for the particular group/station
         :param work_dir: path to the directory where the bluespec files should be generated
-        
-        :type isa_yaml: dict 
-        :type grp_yaml: dict 
-        :type attr_yaml: dict 
+
+        :type isa_yaml: dict
+        :type grp_yaml: dict
+        :type attr_yaml: dict
         :type rvxlen: str
         :type pkg_name: str
         :type last: bool
@@ -1053,7 +1053,7 @@ def station_regs(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkg_name,
     global debug_enabled
     global physical_addr_sz
     global pmp_grain
-        
+
     if logging:
         logger.info('Generating storage elements for : ' + pkg_name)
     bsvstr += (other_vars_temp)
@@ -1140,16 +1140,16 @@ def station_regs(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkg_name,
                         csrname, reset_val)
             elif ('warl' in content[rvxlen]['type']):
                 bsv_str = derive_warl_func( csrname, str(size),
-                        content[rvxlen]['type']['warl']['legal'], 
+                        content[rvxlen]['type']['warl']['legal'],
                         content[rvxlen]['type']['warl']['dependency_fields'])
                 if 'pmpaddr' in csrname:
                     pmpaddrnum = int(csrname.split('pmpaddr')[1],10)
                     pmpcfgnum = int(pmpaddrnum / int(xlen/8))
                     bsvstr += temp_warlReg_pmpaddr.format(description,
-                            physical_addr_sz-2, csrname, reset_val, bsv_str, 
+                            physical_addr_sz-2, csrname, reset_val, bsv_str,
                             pmpcfgnum, pmpaddrnum)
                 else:
-                    bsvstr += temp_warlReg.format(description, str(size), 
+                    bsvstr += temp_warlReg.format(description, str(size),
                             csrname, reset_val, bsv_str)
             else:
                 bsvstr +=  temp_register.format(description, str(xlen), csrname,
@@ -1179,12 +1179,12 @@ def station_regs(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkg_name,
                             subcontent['shadow'].replace('.','_'), type_reg,
                             storage_type])
                     else:
-                        concat.append([ msb, width, csrname + '_' + subfield, 
+                        concat.append([ msb, width, csrname + '_' + subfield,
                          type_reg, storage_type])
 
                     if subfield == 'sd' and csrname == 'mstatus':
                         bsvstr += temp_readOnlyRegister.format(subdescription,
-                                str(width), csrname+'_'+subfield, 
+                                str(width), csrname+'_'+subfield,
                                 "pack((rg_mstatus_xs == 2\'b11) || (rg_mstatus_fs == 2\'b11))")
                     elif subfield in ['meip','msip','mtip']:
                         bsvstr += temp_wire.format(width,csrname+'_'+subfield)
@@ -1239,7 +1239,7 @@ def station_regs(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkg_name,
             bsvstr += attr_yaml[csrname]['action_method']['reg_dec']
             if logging:
                 logger.debug('-- Creating external pending interrupt logic for : ' + csrname)
-        
+
     if logging:
         logger.info('Storage elements created for : ' + pkg_name)
     return bsvstr
@@ -1274,10 +1274,10 @@ def csr_top(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkgname, logging =
         :param attr_yaml: the attributes YAML
         :param rvxlen: a string containing the format: "rv32" or "rv64" depending on the XLEN value of the DUT
         :param pkgname: name of the bluespec package this code belongs to.
-        
-        :type isa_yaml: dict 
-        :type grp_yaml: dict 
-        :type attr_yaml: dict 
+
+        :type isa_yaml: dict
+        :type grp_yaml: dict
+        :type attr_yaml: dict
         :type rvxlen: str
         :type pkgname: str
     '''
@@ -1336,7 +1336,7 @@ def csr_top(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkgname, logging =
         if dpc_grp is None or dcsr_grp_name is None:
             logger.error('Debug is enabled but one/both of dpc, dcsr is not available in any group')
             sys.exit(1)
-    
+
     pmp_entries = 0
 
 
@@ -1349,15 +1349,15 @@ def csr_top(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkgname, logging =
     hypervisor=False
     if None not in (hstatus_grp, htval_grp, htinst_grp, vsepc_grp, vstval_grp, vscause_grp, vstvec_grp):
         hypervisor = True
-        
+
     # imports of each station
     if logging:
         logger.debug('-- Creating imports')
     for groups in grp_yaml:
         bsvstr += 'import csrbox_' + groups + ' :: * ;\n'
     bsvstr += 'import csrbox_decoder :: * ;'
-    
-    
+
+
     if logging:
         logger.debug('-- Creating interface')
     # define a value method for each csr.
@@ -1371,7 +1371,7 @@ def csr_top(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkgname, logging =
             if 'pmpaddr' in csrname:
                 size = physical_addr_sz -2
             bsvstr +=(methodv_dec_temp.format(size, csrname))
-    bsvstr += '''  
+    bsvstr += '''
   endinterface:Sbread'''
 
 
@@ -1406,14 +1406,14 @@ def csr_top(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkgname, logging =
         if logging:
             logger.debug('-- Creating custom action methods for : ' + csrname)
         if attr_yaml[csrname]['action_method']['required'] and \
-                attr_yaml[csrname]['action_method']['global']: 
+                attr_yaml[csrname]['action_method']['global']:
             size = attr_yaml[csrname]['size']
             for arguments in attr_yaml[csrname]['action_method']['def_arguments']:
                 arguments = arguments.replace('xlen',str(xlen))
                 arguments = arguments.replace('size',str(size))
                 if '::' in arguments:
                     cond, args = re.findall(r'^(.*?)::(.*?)$',arguments)[0]
-                    if eval(cond):         
+                    if eval(cond):
                         args = textwrap.indent(args, '')
                     else:
                         args = ''
@@ -1431,7 +1431,7 @@ def csr_top(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkgname, logging =
     bsvstr += (method_top_generic.format(xlen))
     bsvstr += (end_interface_temp)
     # -------------- interface definition done ------------------------------#
-    
+
     #--- create module
     if logging:
         logger.debug('-- Creating module')
@@ -1451,7 +1451,7 @@ def csr_top(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkgname, logging =
     # -- instantiate groups
     for grps in grp_yaml:
         bsvstr += '    let ' + grps + ' <- mk_csrbox_' + grps + ';\n'
-   
+
     bsvstr += '''
     let lv_misa_s = {0}.mv_csr_misa[18];
     let lv_misa_h = {0}.mv_csr_misa[7];
@@ -1463,7 +1463,7 @@ def csr_top(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkgname, logging =
     /*doc:reg: holds the current virtual mode */
     Reg#(Bit#(1)) rg_virtual <- mkReg(0);
 '''.format(misa_grp)
-    
+
     if debug_enabled:
         bsvstr += '''
     /*doc:reg:The register indicates that the core is in debug mode*/
@@ -1481,7 +1481,7 @@ def csr_top(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkgname, logging =
     anydata = anydata[:-3]
     anyupdated = anyupdated[:-3]
     bsvstr += check_hit_temp.format(anyhit, anydata, xlen, anyupdated)
-    
+
     # additional wires/regs
     req_regs = []
     for csrname in isa_yaml:
@@ -1505,14 +1505,14 @@ def csr_top(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkgname, logging =
         logger.debug('-- Creating connections between core-fwd interfaces')
     for i in range(len(grp_list)-1):
         bsvstr += temp_req_connection.format(grp_list[i], grp_list[i+1], str(i))
-    
+
     # -- connect rg_prv to all stations
     for grp in grp_yaml:
         bsvstr += temp_prv_connection.format(grp)
         if debug_enabled:
             bsvstr += temp_dbg_connection.format(grp)
 
-  
+
     # create action methods of dependent csrs
     if logging:
         logger.debug('-- defining action methods')
@@ -1561,7 +1561,7 @@ def csr_top(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkgname, logging =
                         bsvstr += temp_top_csr_connection.format(\
                                     grp, shadow_csr, shadow_grp )
                         incomming_reads.append(shadow_csr)
-            
+
     # return and trap method definitions
     if logging:
         logger.debug('-- Defining trap and return methods')
@@ -1612,7 +1612,7 @@ def csr_top(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkgname, logging =
                 htval_grp, htinst_grp, hstatus_grp, mstatus_grp, mtval_grp,
                 mepc_grp, mcause_grp, mtval2_grp, mtinst_grp, mtvec_grp,
                 debug_trap_str, mstatush_grp)
-    
+
     # define additional required interfaces
     req_ifc = []
     for csrname in isa_yaml:
@@ -1658,14 +1658,14 @@ def csr_top(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkgname, logging =
         bsvstr += pmpcfg_def.format(pmp_entries, pmpcfg_str)
     if hypervisor:
         bsvstr += '''
-        method Action ma_core_req(CSRReq req) ; 
-            req.csr_address = fn_address_virtual(req.csr_address, rg_virtual); 
-            grp1.ma_core_req(req); 
+        method Action ma_core_req(CSRReq req) ;
+            req.csr_address = fn_address_virtual(req.csr_address, rg_virtual);
+            grp1.ma_core_req(req);
         endmethod '''
     else:
         bsvstr += '''
-        method Action ma_core_req(CSRReq req) ; 
-            grp1.ma_core_req(req); 
+        method Action ma_core_req(CSRReq req) ;
+            grp1.ma_core_req(req);
         endmethod '''
     # complete the value method for each csr.
     if logging:
@@ -1679,14 +1679,14 @@ def csr_top(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkgname, logging =
     bsvstr +='''
     endinterface'''
 
-    
+
     # define csr specific action methods.
     if logging:
         logger.debug('-- Defining custom action methods')
     for csrname in isa_yaml:
         logger.debug('Top CSR Interface: ' + csrname)
         if attr_yaml[csrname]['action_method']['required'] and \
-                attr_yaml[csrname]['action_method']['global']: 
+                attr_yaml[csrname]['action_method']['global']:
 #            defs =  attr_yaml[csrname]['action_method']['def_arguments']
             csr_grp = find_group(grp_yaml, csrname.upper())
             for arguments in attr_yaml[csrname]['action_method']['def_arguments']:
@@ -1694,7 +1694,7 @@ def csr_top(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkgname, logging =
                 arguments = arguments.replace('size',str(size))
                 if '::' in arguments:
                     cond, args = re.findall(r'^(.*?)::(.*?)$',arguments)[0]
-                    if eval(cond):         
+                    if eval(cond):
                         args = textwrap.indent(args, '')
                     else:
                         args = ''
@@ -1709,7 +1709,7 @@ def csr_top(isa_yaml, grp_yaml, attr_yaml, aux_info, rvxlen, pkgname, logging =
     # debug specific interfaces
     if debug_enabled:
         bsvstr += method_top_debug_def.format(dcsr_grp_name);
-            
+
 
     # -- end the module
     bsvstr += end_module_temp
